@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2009-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2009-2016 -- leonerd@leonerd.org.uk
 
 package Tickit;
 
@@ -9,21 +9,29 @@ use strict;
 use warnings;
 
 BEGIN {
-   our $VERSION = '0.29_002';
+   our $VERSION = '0.61_001';
 }
 
+use Carp;
+
 use IO::Handle;
 
 use Scalar::Util qw( weaken );
+use Time::HiRes qw( time );
 
 BEGIN {
    require XSLoader;
    XSLoader::load( __PACKAGE__, our $VERSION );
 }
 
-use Tickit::Term;
+use Tickit::Event;
 use Tickit::Window;
 
+use Tickit::Debug;
+
+use Struct::Dumb;
+struct TimeQueue => [qw( time code )];
+
 =head1 NAME
 
 C<Tickit> - Terminal Interface Construction KIT
@@ -75,7 +83,9 @@ eventually the Window layer will be rewritten in XS and C instead.
 
 =cut
 
-=head2 $tickit = Tickit->new( %args )
+=head2 new
+
+   $tickit = Tickit->new( %args )
 
 Constructs a new C<Tickit> framework container object.
 
@@ -102,6 +112,12 @@ C<${^UTF8LOCALE}> variable.
 If defined, sets the root widget using C<set_root_widget> to the one
 specified.
 
+=item use_altscreen => BOOL
+
+If defined but false, disables the use of altscreen, even if supported by the
+terminal. This will mean that the screen contents are stll available after the
+program has finished.
+
 =back
 
 =cut
@@ -120,13 +136,15 @@ sub new
 
    my $self = bless {
       todo_queue => [],
-      redraw_queue => [],
+      timer_queue => [],
    }, $class;
 
    unless( $term ) {
       my $writer = $self->_make_writer( $out );
 
-      $term = Tickit::Term->find_for_term(
+      require Tickit::Term;
+
+      $term = Tickit::Term->new(
          writer        => $writer,
          input_handle  => $in,
          output_handle => $out,
@@ -138,6 +156,8 @@ sub new
    $self->{term_in}  = $in;
    $self->{term_out} = $out;
 
+   $self->{use_altscreen} = $args{use_altscreen} // 1;
+
    my $rootwin = $self->{rootwin} = Tickit::Window->new( $self, $term->lines, $term->cols );
 
    $self->bind_key( 'C-c' => $self->can( "stop" ) );
@@ -146,22 +166,14 @@ sub new
 
    weaken( my $weakself = $self );
 
-   $term->set_on_resize( sub {
+   $term->bind_event( key => sub {
       $weakself or return;
-      my ( $term, $lines, $cols ) = @_;
-      $weakself->rootwin->resize( $lines, $cols );
-   } );
+      my ( $term, $ev, $info ) = @_;
+      Tickit::Debug->log( Ik => "Key event %s %s (mod=%d)",
+         map { $info->$_ } qw( type str mod ) ) if DEBUG;
+      $weakself->on_key( $info );
 
-   $term->set_on_key( sub {
-      $weakself or return;
-      shift; # term
-      $weakself->on_key( @_ );
-   } );
-
-   $term->set_on_mouse( sub {
-      $weakself or return;
-      shift; # term
-      $weakself->on_mouse( @_ );
+      return 1;
    } );
 
    $self->set_root_widget( $args{root} ) if defined $args{root};
@@ -183,7 +195,9 @@ sub _make_writer
    return $out;
 }
 
-=head2 $tickit->later( $code )
+=head2 later
+
+   $tickit->later( $code )
 
 Runs the given CODE reference at some time soon in the future. It will not be
 invoked yet, but will be invoked at some point before the next round of input
@@ -207,33 +221,43 @@ sub later
    push @{ $self->{todo_queue} }, $code;
 }
 
-sub enqueue_redraw
+=head2 timer
+
+ $tickit->timer( at => $epoch, $code )
+
+ $tickit->timer( after => $delay, $code )
+
+Runs the given CODE reference at some fixed point in time in the future.
+The first argmuent must be either the string C<at>, or C<after>; and specifies
+that second argument gives either the absolute epoch time (C<$epoch>), or the
+delay relative to now (C<$delay>), respectively. Fractions are supported to a
+resolution of microseconds.
+
+=cut
+
+sub timer
 {
    my $self = shift;
-   my ( $code ) = @_;
+   my ( $mode, $amount, $code ) = @_;
 
-   my $queue = $self->{redraw_queue};
-   push @$queue, $code if $code;
+   my $at;
+   if( $mode eq "at" ) {
+      $at = $amount;
+   }
+   elsif( $mode eq "after" ) {
+      $at = time + $amount;
+   }
+   else {
+      croak "Mode should be 'at' or 'after'";
+   }
 
-   return if $self->{redraw_queued};
-
-   weaken( my $weakself = $self );
-
-   $self->{redraw_queued} = 1;
-   $self->later( sub {
-      $weakself->term->mode_cursorvis( 0 );
-
-      my @queue = @$queue;
-      @$queue = ();
-
-      $_->() for @queue;
-
-      $weakself->rootwin->restore;
-      $weakself->{redraw_queued} = 0;
-   } );
+   # TODO: bin-search insert position then splice
+   @{ $self->{timer_queue} } = sort { $a->time <=> $b->time } @{ $self->{timer_queue} }, TimeQueue( $at => $code );
 }
 
-=head2 $term = $tickit->term
+=head2 term
+
+   $term = $tickit->term
 
 Returns the underlying L<Tickit::Term> object.
 
@@ -245,9 +269,13 @@ sub term
    return $self->{term};
 }
 
-=head2 $cols = $tickit->cols
+=head2 cols
 
-=head2 $lines = $tickit->lines
+=head2 lines
+
+   $cols = $tickit->cols
+
+   $lines = $tickit->lines
 
 Query the current size of the terminal. Will be cached and updated on receipt
 of C<SIGWINCH> signals.
@@ -267,16 +295,18 @@ sub _SIGWINCH
 sub on_key
 {
    my $self = shift;
-   my ( $type, $str, $mod ) = @_;
+   my ( $info ) = @_;
 
-   $self->rootwin->_handle_key( @_ ) and return;
+   my $str = $info->str;
 
    if( exists $self->{key_binds}{$str} ) {
       $self->{key_binds}{$str}->( $self, $str ) and return;
    }
 }
 
-=head2 $tickit->bind_key( $key, $code )
+=head2 bind_key
+
+   $tickit->bind_key( $key, $code )
 
 Installs a callback to invoke if the given key is pressed, overwriting any
 previous callback for the same key. The code block is invoked as
@@ -306,14 +336,9 @@ sub bind_key
    }
 }
 
-sub on_mouse
-{
-   my $self = shift;
+=head2 rootwin
 
-   $self->rootwin->_handle_mouse( @_ ) and return;
-}
-
-=head2 $tickit->rootwin
+   $tickit->rootwin
 
 Returns the root L<Tickit::Window>.
 
@@ -325,7 +350,9 @@ sub rootwin
    return $self->{rootwin};
 }
 
-=head2 $tickit->set_root_widget( $widget )
+=head2 set_root_widget
+
+   $tickit->set_root_widget( $widget )
 
 Sets the root widget for the application's display. This must be a subclass of
 L<Tickit::Widget>.
@@ -338,7 +365,9 @@ sub set_root_widget
    ( $self->{root_widget} ) = @_;
 }
 
-=head2 $tickit->setup_term
+=head2 setup_term
+
+   $tickit->setup_term
 
 Set up the screen and generally prepare to start running
 
@@ -349,9 +378,12 @@ sub setup_term
    my $self = shift;
 
    my $term = $self->term;
-   $term->mode_altscreen( 1 );
-   $term->mode_cursorvis( 0 );
-   $term->mode_mouse( 1 );
+
+   $term->await_started( 0.100 ); # 100 msec
+
+   $term->setctl_int( altscreen => 1 ) if $self->{use_altscreen};
+   $term->setctl_int( cursorvis => 0 );
+   $term->setctl_int( mouse     => Tickit::Term::TERM_MOUSEMODE_DRAG );
    $term->clear;
 
    if( my $widget = $self->{root_widget} ) {
@@ -361,7 +393,9 @@ sub setup_term
    $term->flush;
 }
 
-=head2 $tickit->teardown_term
+=head2 teardown_term
+
+   $tickit->teardown_term
 
 Shut down the screen after running
 
@@ -376,14 +410,23 @@ sub teardown_term
    }
 
    my $term = $self->term;
-   $term->mode_altscreen( 0 );
-   $term->mode_cursorvis( 1 );
-   $term->mode_mouse( 0 );
+
+   if( $self->{use_altscreen} ) {
+      $term->setctl_int( altscreen => 0 );
+   }
+   else {
+      $term->goto( $term->get_size );
+   }
+
+   $term->setctl_int( cursorvis => 1 );
+   $term->setctl_int( mouse     => 0 );
 
    $term->flush;
 }
 
-=head2 $tickit->tick
+=head2 tick
+
+   $tickit->tick
 
 Run a single round of IO events. Does not call C<setup_term> or
 C<teardown_term>.
@@ -394,7 +437,19 @@ sub _tick
 {
    my $self = shift;
 
-   $self->{term}->input_wait;
+   my $timer_queue = $self->{timer_queue};
+
+   my $timeout;
+   if( @$timer_queue ) {
+      $timeout = $self->{timer_queue}[0]->time - time;
+   }
+
+   $self->{term}->input_wait( $timeout );
+
+   my $now = time;
+   while( @$timer_queue and $timer_queue->[0]->time <= $now ) {
+      shift( @$timer_queue )->code->();
+   }
 
    $self->_flush_later if @{ $self->{todo_queue} };
 }
@@ -416,7 +471,9 @@ sub tick
    $self->_tick;
 }
 
-=head2 $tickit->run
+=head2 run
+
+   $tickit->run
 
 Calls the C<setup_term> method, then processes IO events until stopped, by the
 C<stop> method, C<SIGINT>, C<SIGTERM> or the C<Ctrl-C> key. Then runs the
@@ -430,7 +487,17 @@ sub run
 
    $self->setup_term;
 
-   $SIG{INT} = $SIG{TERM} = sub { $self->stop };
+   $SIG{INT} = $SIG{TERM} = sub {
+      my $signal = shift;
+
+      # Disable the handler so a second signal will be fatal
+      undef $SIG{$signal};
+      $SIG{$signal} = sub {
+         die "Aborting Tickit on double SIGINT\n";
+      };
+
+      $self->stop;
+   };
 
    $SIG{WINCH} = sub {
       $self->later( sub { $self->_SIGWINCH } )
@@ -454,7 +521,9 @@ sub run
    $self->teardown_term;
 }
 
-=head2 $tickit->stop
+=head2 stop
+
+   $tickit->stop
 
 Causes a currently-running C<run> method to stop processing events and return.
 
