@@ -1,7 +1,7 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2016 -- leonerd@leonerd.org.uk
 
 package Tickit::Rect;
 
@@ -10,7 +10,7 @@ use warnings;
 
 use Carp;
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.61_001';
 
 # Load the XS code
 require Tickit;
@@ -40,12 +40,14 @@ simply as a convenient data store containing some useful utility methods.
 
 =cut
 
-=head2 $rect = Tickit::Rect->new( %args )
+=head2 new
+
+   $rect = Tickit::Rect->new( %args )
 
 Construct a new rectangle of the given geometry, given by C<top>, C<left> and
 either C<lines> and C<cols>, or C<bottom> and C<right>.
 
-=head2 $rect = Tickit::Rect->new( $str )
+   $rect = Tickit::Rect->new( $str )
 
 If given a single string, this will be parsed in the form
 
@@ -71,14 +73,18 @@ sub new
    return $class->_new( @args{qw( top left lines cols )} );
 }
 
-=head2 $rect = $existing_rect->intersect( $other_rect )
+=head2 intersect
+
+   $rect = $existing_rect->intersect( $other_rect )
 
 If there is an intersection between the given rectangles, return it. If not,
 return C<undef>.
 
 =cut
 
-=head2 $rect = $existing_rect->translate( $downward, $rightward )
+=head2 translate
+
+   $rect = $existing_rect->translate( $downward, $rightward )
 
 Returns a new rectangle of the same size as the given one, moved down and to
 the right by the given argmuents (which may be negative)
@@ -89,70 +95,111 @@ the right by the given argmuents (which may be negative)
 
 =cut
 
-=head2 $top = $rect->top
+=head2 top
 
-=head2 $left = $rect->left
+=head2 left
 
-=head2 $bottom = $rect->bottom
+=head2 bottom
 
-=head2 $right = $rect->right
+=head2 right
+
+   $top = $rect->top
+
+   $left = $rect->left
+
+   $bottom = $rect->bottom
+
+   $right = $rect->right
 
 Return the edge boundaries of the rectangle.
 
-=head2 $lines = $rect->lines
+=head2 lines
 
-=head2 $cols = $rect->cols
+=head2 cols
+
+   $lines = $rect->lines
+
+   $cols = $rect->cols
 
 Return the size of the rectangle.
 
 =cut
 
-=head2 @lines = $rect->linerange
+=head2 linerange
 
-A convenient shortcut to generate the list of lines covered; being
+   @lines = $rect->linerange( $min, $max )
 
- $rect->top .. $rect->bottom - 1
+A convenient shortcut to generate the list of lines covered that are within
+the given bounds (either bound may be given as C<undef>). Without bounds,
+equivalent to:
+
+   $rect->top .. $rect->bottom - 1
 
 =cut
 
 sub linerange
 {
    my $self = shift;
-   return $self->top .. $self->bottom - 1;
+   my ( $min, $max ) = @_;
+
+   my $start = $self->top;
+   $start = $min if defined $min and $min > $start;
+
+   my $stop = $self->bottom - 1;
+   $stop = $max if defined $max and $max < $stop;
+
+   return $start .. $stop;
 }
 
 =head1 METHODS
 
 =cut
 
-=head2 $bool = $rect->equals( $other )
+=head2 equals
 
-=head2 $bool = ( $rect == $other )
+   $bool = $rect->equals( $other )
 
-Returns true if C<$other> represents the same area as C<$rect>.
+   $bool = ( $rect == $other )
+
+Returns true if C<$other> represents the same area as C<$rect>. This method
+overloads the numerical equality operator (C<==>).
 
 =cut
 
 use overload '==' => "equals", eq => "equals";
 
-=head2 $bool = $rect->contains( $other )
+=head2 contains
+
+   $bool = $rect->contains( $other )
 
 Returns true if C<$other> is entirely contained within the bounds of C<$rect>.
 
 =cut
 
-=head2 $bool = $rect->intersects( $other )
+=head2 intersects
+
+   $bool = $rect->intersects( $other )
 
 Returns true if C<$other> and C<$rect> intersect at all, even if they overlap.
 
 =cut
 
-use overload '""' => sub {
+sub sprintf
+{
    my $self = shift;
-   sprintf "Tickit::Rect[(%d,%d)..(%d,%d)]", $self->left, $self->top, $self->right, $self->bottom;
-};
+   return sprintf "[(%d,%d)..(%d,%d)]", $self->left, $self->top, $self->right, $self->bottom;
+}
 
-=head2 @r = $rect->add( $other )
+use overload
+   '""' => sub {
+      my $self = shift;
+      return ref($self) . $self->sprintf;
+   },
+   bool => sub { 1 };
+
+=head2 add
+
+   @r = $rect->add( $other )
 
 Returns a list of the non-overlapping regions covered by either C<$rect> or
 C<$other>.
@@ -164,7 +211,9 @@ the original two. This list will contain anywhere between 1 and 3 rectangles.
 
 =cut
 
-=head2 @r = $rect->subtract( $other )
+=head2 subtract
+
+   @r = $rect->subtract( $other )
 
 Returns a list of the non-overlapping regions covered by C<$rect> but not by
 C<$other>.
