@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2012 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2012-2016 -- leonerd@leonerd.org.uk
 
 package Tickit::RectSet;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.61_001';
 
 use List::Util qw( min max );
 
@@ -36,7 +36,9 @@ left-to-right order.
 
 =cut
 
-=head2 $rectset = Tickit::RectSet->new
+=head2 new
+
+   $rectset = Tickit::RectSet->new
 
 Returns a new C<Tickit::RectSet> instance, initially empty.
 
@@ -46,39 +48,51 @@ Returns a new C<Tickit::RectSet> instance, initially empty.
 
 =cut
 
-=head2 @rects = $rectset->rects
+=head2 rects
+
+   @rects = $rectset->rects
 
 Returns a list of the covered regions, in order first top to bottom, then left
 to right.
 
 =cut
 
-=head2 $rectset->add( $rect )
+=head2 add
+
+   $rectset->add( $rect )
 
 Adds the region covered by C<$rect> to the stored region list.
 
 =cut
 
-=head2 $rectset->subtract( $rect )
+=head2 subtract
+
+   $rectset->subtract( $rect )
 
 Removes any covered region that intersects with C<$rect> from the stored
 region list.
 
 =cut
 
-=head2 $rectset->clear
+=head2 clear
+
+   $rectset->clear
 
 Remove all the regions from the set.
 
 =cut
 
-=head2 $bool = $rectset->intersects( $rect )
+=head2 intersects
+
+   $bool = $rectset->intersects( $rect )
 
 Returns true if C<$rect> intersects with any region in the set.
 
 =cut
 
-=head2 $bool = $rectset->contains( $rect )
+=head2 contains
+
+   $bool = $rectset->contains( $rect )
 
 Returns true if C<$rect> is entirely covered by the regions in the set. Note
 that it may be that the rect requires two or more regions in the set to
