@@ -1,14 +1,16 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2016 -- leonerd@leonerd.org.uk
 
 package Tickit::Test;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.61_001';
+
+use Carp;
 
 use Exporter 'import';
 
@@ -18,6 +20,7 @@ our @EXPORT = qw(
    mk_term_and_window
    flush_tickit
    drain_termlog
+   clear_term
 
    resize_term
 
@@ -48,6 +51,8 @@ use Tickit::Test::MockTerm;
 use Tickit::Pen;
 use Tickit;
 
+use Tickit::Utils qw( textwidth substrwidth );
+
 use Test::Builder;
 
 =head1 NAME
@@ -98,7 +103,9 @@ installable module, so that authors of widget subclasses can use it too.
 my $term;
 my $tickit;
 
-=head2 $term = mk_term
+=head2 mk_term
+
+   $term = mk_term
 
 Constructs and returns the mock terminal to unit test with. This object will
 be cached and returned if this function is called again. Most unit tests will
@@ -116,7 +123,9 @@ sub mk_term
    return $term ||= Tickit::Test::MockTerm->new( @_ );
 }
 
-=head2 $win = mk_window
+=head2 mk_window
+
+   $win = mk_window
 
 Construct a root window using the mock terminal, to unit test with.
 
@@ -135,12 +144,14 @@ sub mk_window
    $tickit->setup_term;
 
    # Clear the method log from ->setup_term
-   $term->methodlog;
+   $term->get_methodlog;
 
    return $win;
 }
 
-=head2 ( $term, $win ) = mk_term_and_window
+=head2 mk_term_and_window
+
+   ( $term, $win ) = mk_term_and_window
 
 Constructs and returns the mock terminal and root window; equivalent to
 calling each of C<mk_term> and C<mk_window> separately.
@@ -167,6 +178,8 @@ sub cols  { return $term->cols  }
 
 =head2 flush_tickit
 
+   flush_tickit
+
 Flushes any pending C<later> events in the testing C<Tickit> object. Because
 the unit test script has no real event loop, this is required instead, to
 flush any pending events.
@@ -179,10 +192,14 @@ sub flush_tickit
       my @queue = @later; @later = ();
       $_->() for @queue;
    }
+
+   $tickit->rootwin->flush if $tickit && $tickit->rootwin;
 }
 
 =head2 drain_termlog
 
+   drain_termlog
+
 Drains any pending events from the method log used by the C<is_termlog> test.
 Useful to clear up non-tested events before running a test.
 
@@ -190,10 +207,27 @@ Useful to clear up non-tested events before running a test.
 
 sub drain_termlog
 {
-   $term->methodlog;
+   $term->get_methodlog;
 }
 
-=head2 resize_term( $lines, $cols )
+=head2 clear_term
+
+   clear_term
+
+Clears the entire content form the mock terminal. Useful at the end of a
+section of tests before starting another one. Don't forget to C<drain_termlog>
+afterwards.
+
+=cut
+
+sub clear_term
+{
+   $term->clear
+}
+
+=head2 resize_term
+
+   resize_term( $lines, $cols )
 
 Resize the virtual testing terminal to the size given
 
@@ -203,12 +237,11 @@ sub resize_term
 {
    my ( $lines, $cols ) = @_;
    $term->resize( $lines, $cols );
-
-   # This is evil hackery
-   $tickit->rootwin->resize( $tickit->lines, $tickit->cols );
 }
 
-=head2 presskey( $type, $str, $mod )
+=head2 presskey
+
+   presskey( $type, $str, $mod )
 
 Fire a key event
 
@@ -218,10 +251,12 @@ sub presskey
 {
    my ( $type, $str, $mod ) = @_;
 
-   $tickit->on_key( $type, $str, $mod||0 );
+   $term->_emit_key( Tickit::Event::Key->_new( $type, $str, $mod || 0 ) );
 }
 
-=head2 pressmouse( $ev, $button, $line, $col, $mod )
+=head2 pressmouse
+
+   pressmouse( $type, $button, $line, $col, $mod )
 
 Fire a mouse button event
 
@@ -229,14 +264,14 @@ Fire a mouse button event
 
 sub pressmouse
 {
-   my ( $ev, $button, $line, $col, $mod ) = @_;
+   my ( $type, $button, $line, $col, $mod ) = @_;
 
-   $tickit->on_mouse( $ev, $button, $line, $col, $mod||0 );
+   $term->_emit_mouse( Tickit::Event::Mouse->_new( $type, $button, $line, $col, $mod || 0 ) );
 }
 
 =head1 TEST FUNCTIONS
 
-The following functions can be used like C<Test::More> primatives, in unit
+The following functions can be used like C<Test::More> primitives, in unit
 test scripts.
 
 =cut
@@ -244,10 +279,21 @@ test scripts.
 sub _pen2string
 {
    my $pen = shift;
-   return "{" . join( ",", map { defined $pen->{$_} ? "$_=" . ($pen->{$_} || 0) : "!$_" } sort keys %$pen ) . "}";
+   my %attrs = $pen ? %$pen : ();
+
+   # Normalise requests to reset to default as undef
+   defined $attrs{$_} and $attrs{$_} == -1 and undef $attrs{$_} for @Tickit::Pen::INT_ATTRS;
+   !$attrs{$_}                             and undef $attrs{$_} for @Tickit::Pen::BOOL_ATTRS;
+
+   # Remove undefs
+   defined $attrs{$_} or delete $attrs{$_} for keys %attrs;
+
+   return "{" . join( ",", map { defined $attrs{$_} ? "$_=" . ($attrs{$_} || 0) : "!$_" } sort keys %attrs ) . "}";
 }
 
-=head2 is_termlog( $log, $name )
+=head2 is_termlog
+
+   is_termlog( [ @log ], $name )
 
 Asserts that the mock terminal log contains exactly the given sequence of
 methods. See also the helper functions below.
@@ -256,43 +302,66 @@ Because this test is quite fragile, relying on the exact nature and order of
 drawing methods invoked on the terminal, it should only be used rarely. Most
 normal cases of widget unit tests should instead only use C<is_display>.
 
+   is_termlog( { $pos => \@log, ... }, $name )
+
+The expectation HASH is keyed by strings giving a GOTO position, and the test
+asserts that a sequence of GOTO and other operations were performed equivalent
+to the expectations given in the HASH.
+
+This differs from the simpler ARRAY reference form by being somewhat more
+robust against rendering order. It checks that every expectation sequence
+happens exactly once, but does not care which order the sections happen in.
+
+ is_termlog( { "0,0" => [ PRINT("Hello") ],
+               "0,6" => [ PRINT("World!") ] } );
+
 =cut
 
-sub is_termlog
+sub _step_to_text
 {
-   my ( $log, $name ) = @_;
+   my ( $step ) = @_;
 
-   my $tb = Test::Builder->new;
+   return "none" unless defined $step;
 
-   my @want_log = @$log;
-   my @got_log  = $term->methodlog;
+   my ( $op, @args ) = @$step;
+
+   if( $op eq "setpen" ) {
+      return "$op(" . _pen2string( $args[0] ) . ")";
+   }
+   else {
+      return "$op(" . join( ",", map { defined $_ ? $_ =~ m/^-?\d+$/ ? $_ : qq("$_") : "undef" } @args ) . ")";
+   }
+}
+
+sub _steps_ok
+{
+   my ( $tb, $want_log, $got_log, $stop_before_GOTO, $name ) = @_;
 
    my $prev_line;
 
-   for( my $idx = 0; @want_log or @got_log; $idx++ ) {
-      my $want_line = shift @want_log;
-      my $got_line  = shift @got_log;
+   for( my $idx = 0; @$want_log or @$got_log; $idx++ ) {
+      my $got_line;
 
-      if( $want_line and $want_line->[0] eq "chpen_bg" and
-          $got_line  and $got_line->[0] eq "chpen" ) {
-         $got_line = [ chpen_bg => $got_line->[1]->{bg} ];
+      if( $stop_before_GOTO and @$got_log and $got_log->[0][0] eq "goto" ) {
+         $got_line = undef;
+      }
+      else {
+         $got_line = shift @$got_log;
       }
 
-      foreach ( $want_line, $got_line ) {
-         ( $_ = "none" ), next unless defined $_;
+      my $want_line = shift @$want_log;
 
-         my ( $op, @args ) = @$_;
-
-         if( $op eq "chpen" ) {
-            $_ = "$op(" . _pen2string( $args[0] ) . ")";
-         }
-         else {
-            $_ = "$op(" . join( ",", map { defined $_ ? $_ =~ m/^-?\d+$/ ? $_ : qq("$_") : "undef" } @args ) . ")";
-         }
+      if( $want_line and $want_line->[0] eq "setpen_bg" and
+          $got_line  and $got_line->[0] eq "setpen" ) {
+         $got_line = [ setpen_bg => $got_line->[1]->{bg} ];
       }
 
+      $_ = _step_to_text($_) for $want_line, $got_line;
+
       if( $want_line eq $got_line ) {
          $prev_line = $want_line;
+
+         return 1 if $stop_before_GOTO and @$got_log and $got_log->[0][0] eq "goto";
          next;
       }
 
@@ -303,10 +372,61 @@ sub is_termlog
       return $ok;
    }
 
+   return 1;
+}
+
+sub is_termlog
+{
+   my ( $log, $name ) = @_;
+
+   my $tb = Test::Builder->new;
+
+   my @got_log = $term->get_methodlog;
+
+   if( ref $log eq "ARRAY" ) {
+      local $Test::Builder::Level = $Test::Builder::Level + 1;
+      return unless _steps_ok( $tb, $log, \@got_log, 0, $name );
+   }
+   elsif( ref $log eq "HASH" ) {
+      my %regions = %$log;
+
+      while( keys %regions and @got_log ) {
+         if( !$got_log[0]->[0] eq "goto" ) {
+            my $ok = $tb->ok( 0, $name );
+            $tb->diag( "Expected a goto terminal operation, got " . _step_to_text( $got_log[0] ) );
+            return $ok;
+         }
+
+         my $pos = sprintf "%d,%d", @{ shift @got_log }[1,2];
+         my $want_log = delete $regions{$pos};
+         unless( $want_log ) {
+            my $ok = $tb->ok( 0, $name );
+            $tb->diag( "Did not expect goto($pos)" );
+            return $ok;
+         }
+
+         local $Test::Builder::Level = $Test::Builder::Level + 1;
+         return unless _steps_ok( $tb, $want_log, \@got_log, 1, $name );
+      }
+
+      if( keys %regions ) {
+         my $ok = $tb->ok( 0, $name );
+         $tb->diag( "Expected a goto(" . ( keys %regions )[0] . ", got none" );
+         return $ok;
+      }
+      if( @got_log ) {
+         my $ok = $tb->ok( 0, $name );
+         $tb->diag( "Expected none, got " . _step_to_text( $got_log[0] ) );
+         return $ok;
+      }
+   }
+
    return $tb->ok( 1, $name );
 }
 
-=head2 is_display( $lines, $name )
+=head2 is_display
+
+   is_display( $lines, $name )
 
 Asserts that the mock terminal display is exactly that as given by the content
 of C<$lines>, which must be an ARRAY reference containing one value for each
@@ -366,7 +486,7 @@ sub is_display
 
             $want_text .= " " x ( $term->cols - $col ) unless defined $want_text and length $want_text;
 
-            my $got_text = $term->get_display_text( $line, $col, length $want_text );
+            my $got_text = $term->get_display_text( $line, $col, textwidth $want_text );
             if( $got_text ne $want_text ) {
                my $ok = $tb->ok( 0, $name );
                $tb->diag( "Display differs on line $line at column $col" );
@@ -377,8 +497,8 @@ sub is_display
 
             my $want_pen = _pen2string( $chunk->[1] );
             my $idx = 0;
-            while( $idx < length $want_text ) {
-               if( substr( $want_text, $idx, 1 ) eq " " ) {
+            while( $idx < textwidth $want_text ) {
+               if( substrwidth( $want_text, $idx, 1 ) eq " " ) {
                   my $want_bg = $chunk->[1]->{bg} // "undef";
                   my $got_bg = $term->get_display_pen( $line, $col )->{bg} // "undef";
                   if( $got_bg ne $want_bg ) {
@@ -432,7 +552,9 @@ sub is_display
    return $tb->ok( 1, $name );
 }
 
-=head2 is_cursorpos( $line, $col, $name )
+=head2 is_cursorpos
+
+   is_cursorpos( $line, $col, $name )
 
 Asserts that the mock terminal cursor is at the given position.
 
@@ -444,7 +566,8 @@ sub is_cursorpos
 
    my $tb = Test::Builder->new;
 
-   my ( $at_line, $at_col ) = $term->get_position;
+   my $at_line = $term->line;
+   my $at_col  = $term->col;
 
    my $ok = $tb->ok( $line == $at_line && $col == $at_col, $name );
 
@@ -488,8 +611,6 @@ to C<is_termlog>.
  CLEAR
  GOTO($line,$col)
  ERASECH($count,$move_to_end)
- INSERTCH($count)
- DELETECH($count)
  SCROLLRECT($top,$left,$lines,$cols,$downward,$rightward)
  PRINT($string)
  SETPEN(%attrs)
@@ -500,12 +621,16 @@ to C<is_termlog>.
 sub CLEAR      { [ clear => ] }
 sub GOTO       { [ goto => $_[0], $_[1] ] }
 sub ERASECH    { [ erasech => $_[0], $_[1] || 0 ] }
-sub INSERTCH   { [ insertch => $_[0] ] }
-sub DELETECH   { [ deletech => $_[0] ] }
 sub SCROLLRECT { [ scrollrect => @_[0..5] ] }
 sub PRINT      { [ print => $_[0] ] }
-sub SETPEN     { [ chpen => { DEFAULTPEN, @_ } ] }
-sub SETBG      { [ chpen_bg => $_[0] ] }
+sub SETPEN     { [ setpen => { DEFAULTPEN, @_ } ] }
+sub SETBG      { [ setpen_bg => $_[0] ] }
+
+# Deprecated, will never match now
+sub INSERTCH   { carp "INSERTCH() is no longer used by MockTerm";
+                 [ insertch => $_[0] ] }
+sub DELETECH   { carp "DELETECH() is no longer used by MockTerm";
+                 [ deletech => $_[0] ] }
 
 =head1 AUTHOR
 
