@@ -1,14 +1,14 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2011-2013 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2011-2016 -- leonerd@leonerd.org.uk
 
 package Tickit::Utils;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.29_002';
+our $VERSION = '0.61_001';
 
 use Carp;
 
@@ -46,22 +46,25 @@ This module provides a number of utility functions used across C<Tickit>.
 
 =head1 FUNCTIONS
 
-=head2 $bytes = string_count( $str, $pos, $limit )
+=head2 string_count
+
+   $bytes = string_count( $str, $pos, $limit )
 
 Given a string in C<$str> and a L<Tickit::StringPos> instance in C<$pos>,
 updates the counters in C<$pos> by counting the string, and returns the number
 of bytes consumed. If C<$limit> is given, then it will count no further than
 any of the limits given.
 
-=head2 $bytes = string_countmore( $str, $pos, $limit, $start )
+=head2 string_countmore
+
+   $bytes = string_countmore( $str, $pos, $limit )
 
 Similar to C<string_count> but will not zero the counters before it begins.
-Counters in C<$pos> will still be incremented. If C<$start> is provided it
-gives the byte offset within C<$str> to begin counting from. This is more
-efficient than applying C<substr> on the input string to create the starting
-point.
+Counters in C<$pos> will still be incremented.
 
-=head2 $cols = textwidth( $str )
+=head2 textwidth
+
+   $cols = textwidth( $str )
 
 Returns the number of screen columns consumed by the given (Unicode) string.
 
@@ -69,7 +72,9 @@ Returns the number of screen columns consumed by the given (Unicode) string.
 
 # Provided by XS
 
-=head2 @cols = chars2cols( $text, @chars )
+=head2 chars2cols
+
+   @cols = chars2cols( $text, @chars )
 
 Given a list of increasing character positions, returns a list of column
 widths of those characters. In scalar context returns the first columns width.
@@ -78,7 +83,9 @@ widths of those characters. In scalar context returns the first columns width.
 
 # Provided by XS
 
-=head2 @chars = cols2chars( $text, @cols )
+=head2 cols2chars
+
+   @chars = cols2chars( $text, @cols )
 
 Given a list of increasing column widths, returns a list of character
 positions at those widths. In scalar context returns the first character
@@ -88,11 +95,13 @@ position.
 
 # Provided by XS
 
-=head2 $substr = substrwidth $text, $startcol
+=head2 substrwidth
 
-=head2 $substr = substrwidth $text, $startcol, $widthcols
+   $substr = substrwidth $text, $startcol
 
-=head2 $substr = substrwidth $text, $startcol, $widthcols, $replacement
+   $substr = substrwidth $text, $startcol, $widthcols
+
+   $substr = substrwidth $text, $startcol, $widthcols, $replacement
 
 Similar to C<substr>, but counts start offset and length in screen columns
 instead of characters
@@ -116,7 +125,9 @@ sub substrwidth
    }
 }
 
-=head2 ( $before, $alloc, $after ) = align( $value, $total, $alignment )
+=head2 align
+
+   ( $before, $alloc, $after ) = align( $value, $total, $alignment )
 
 Returns a list of three integers created by aligning the C<$value> to a
 position within the C<$total> according to C<$alignment>. The sum of the three
@@ -144,7 +155,9 @@ sub align
    return ( $before, $value, $spare - $before );
 }
 
-=head2 $val = bound( $min, $val, $max )
+=head2 bound
+
+   $val = bound( $min, $val, $max )
 
 Returns the value of C<$val> bounded by the given minimum and maximum. Either
 limit may be left undefined, causing no limit of that kind to be applied.
@@ -159,7 +172,9 @@ sub bound
    return $val;
 }
 
-=head2 distribute( $total, @buckets )
+=head2 distribute
+
+   distribute( $total, @buckets )
 
 Given a total amount of quota, and a list of buckets, distributes the quota
 among the buckets according to the values given in them.
@@ -292,7 +307,7 @@ sub distribute
       foreach my $b ( @buckets ) {
          my $amount;
 
-         if( $b->{base} ) {
+         if( defined $b->{base} ) {
             $amount = $b->{base} * $allocatable / $base_total;
 
             $err += $amount - int($amount);
@@ -300,7 +315,7 @@ sub distribute
 
             $amount = int($amount);
          }
-         elsif( $b->{fixed} ) {
+         elsif( defined $b->{fixed} ) {
             $amount = $b->{fixed};
          }
 
